package Blog;
#
# phroggy.com 5.4
#
# (C) 2009 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These are functions for dealing with the weblog.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    LoadBlog
    GetBlogInfo
    GetBlogList
    GetBlogIndex
    GetRecentBlog
    BlogSearch
    ReadBlogCache
    RebuildBlogCache
  );
}

use strict;
use warnings;
use PhroggyIO;
use PhroggyHTML;
use Paths;
use Localize;
use Navigation;

our %BlogCache;
our @BlogIndex;

sub LoadBlog {
  # Arguments: blog number
  # Returns: Page object
  
  my($blog)=@_;
  my($box)=load DocBox("weblog/$blog");
  if($box->Success()) {
    my($nav,%links)=_GetBlogNav($blog);
    my $page=new Page('Title'=>'phroggy.com: '.Translate('Weblog Archive').': '.$box->Title(),
                      'Filename'=>"weblog-$blog",'LinkUp'=>'weblog');
    $page->Set(%links);
    $page->AddBox($nav,$box,$nav);
    $page->AddHead('<link rel="alternate" href="'.$Paths::Web{'base'}.'/rss.pl" type="application/rss+xml" title="Phroggy.com RSS Feed">');
    return($page);
  } else {
    return load Page('filenotfound');
  }  
}

sub GetBlogInfo {
  # Arguments: blog number
  # Returns: title, numeric date
  
  my($blog)=@_;
  ReadBlogCache() unless(%BlogCache);
  return($BlogCache{$blog}{'Title'},$BlogCache{$blog}{'Date'});
}

sub GetBlogList {
  # Returns: sorted list of all blog numbers
  
  ReadBlogCache() unless(%BlogCache);
  my @list=();
  return sort {$a <=> $b} keys %BlogCache;
##  return sort {$BlogCache{$a}{'Date'} <=> $BlogCache{$b}{'Date'}} keys %BlogCache;
}

sub GetBlogIndex {
  # Returns:  Box object, UNLESS there's a single result

  my($searchhtml,$status,@filenames)=BlogSearch($Browser::Query{'q'});
  if($Browser::Query{'q'} && @filenames==1) {
    print "Status: 302 Found\n";
    print "Location: ".GetURI($filenames[0])."\n\n";
    exit;
  }
  my $localsearch=Translate('Search');
  my $placeholder=Translate('Enter keywords');
  
  my $body=<<"EOF";
<form action="{URI}" method="get">
	<label for="search_string">$localsearch:</label>
	<input id="search_string" name="q" type="search" value="{SEARCH_TERMS}"
		placeholder="$placeholder" autosave="com.phroggy.weblog" results=10 incremental
		onSearch="updateSearch('{CGIPATH}/blogsearch.pl');">
	<span id="search_status">$status</span>
</form>

<hr>

<div id="search_results">
$searchhtml
</div>
EOF
  return new DocBox('Title'=>Translate('Weblog Index'),'Body'=>$body);
}

sub GetRecentBlog {
  # Returns: array of Box objects
  
  my $NUM=3; # how many entries to show
  
  my @list=GetBlogList();
  my @boxes=();
  for(my $i=$#list;$i>-1 && $i>$#list-$NUM;$i--) {
    push(@boxes,load DocBox("weblog/$list[$i]"));
  }
  return(@boxes);
}

sub BlogSearch {
  # Arguments:  search string
  # Returns:  HTML results blob, HTML status or "", list of matching filenames
  
  my($search)=@_;
  ReadBlogCache();
  
  my %Matches=();
  my $showall=1;
  my $stripped=(defined $search)?lc $search:'';
  $stripped=~s/\x{2014}/ /g; # replace em-dash with space (this should break words)
  $stripped=~s/[^ \-\d\w\xc0-\x{ffff}]//g; # strip punctuation characters (except hyphen)
  my @words=split /\s+/,$stripped;
  if(@words) {
    $showall=0;
    # Build a hash of every blog entry that matches ANY search terms
    my %firstrun=();
    open(DB,"$Paths::File{'data'}/blogwordcache.data") or return qq[<span style="color: #f00;">Cannot read blog word cache.</span>\n],'';
    while(<DB>) {
      chomp;
      next unless(/^(\S+): (.*)$/);
      foreach my $word (@words) {
        next if(index($1,$word)==-1);
        foreach my $blog (split /,/,$2) {
          $firstrun{$word}{$blog}=1;
          $Matches{$blog}=0;
        }
      }
    }
    close DB;
    # Count how many search terms are matched by each result
    foreach my $word (keys %firstrun) {
      $Matches{$_}++ foreach (keys %{$firstrun{$word}});
    }
  } elsif($search) {
    # User entered something, but after stripping extra characters there was nothing left
    return qq[<span style="color: #f00;">Invalid search term.</span>\n],'';
  }
  
  my @results=();
  foreach my $blog (@Blog::BlogIndex) {
    # Omit anything that doesn't match ALL search terms
    next unless($showall || $Matches{$blog} && $Matches{$blog}==@words);
    # Omit localized versions that don't match the best language for each blog entry
    my($filename,$language)=split /_/,$blog;
    $language='' unless(defined($language));
    next unless($Blog::BlogCache{$filename}{'Language'} eq $language);
    # Build results list
    my $title=$Blog::BlogCache{$filename}{'Title'};
    my $date=$Blog::BlogCache{$filename}{'Date'};
    push @results,{'Filename'=>$filename,'Title'=>$title,'Date'=>$date};
  }
  
  # Return results
  my $html='';
  my @filenames=();
  foreach my $blog (sort {$a->{'Filename'} <=> $b->{'Filename'}} @results) {
    my($filename,$title,$date)=($blog->{'Filename'},Escape($blog->{'Title'}),ShortDate($blog->{'Date'}));
    push @filenames,"weblog-$filename";
    $date=$date?" ($date)":'';
    my $uri=GetURI("weblog-$filename");
    # FIXME - append original search term to linked URL as ?q=whatever (requires special escaping)
    $html.=qq[<a href="$uri" title="Weblog entry #$filename: $title$date">\n];
    $html.=qq[$title</a>$date<br>\n];
  }
  return $html,$showall?'':SearchMatches(scalar @filenames),@filenames;
}

sub ReadBlogCache {
  %BlogCache=();
  @BlogIndex=();
  my(%Title,%Date);

  my $cachetime=0;
  if(-f "$Paths::File{'data'}/blogmetacache.data") {
    $cachetime=(stat "$Paths::File{'data'}/blogmetacache.data")[9];
  }
  my $dirtime=(stat "$Paths::File{'docs'}/weblog")[9];
  RebuildBlogCache() if($cachetime<$dirtime);
  
  open(DB,"$Paths::File{'data'}/blogmetacache.data") or Abort('Cannot read blog metadata cache.');
  while(<DB>) {
    chomp;
    my($blog,$date,$title)=split /\t/;
    my($filename,$language)=split /_/,$blog;
    $language='' unless(defined($language));
    $Date{$filename}{$language}=$date;
    $Title{$filename}{$language}=$title;
  }
  close DB;
  foreach my $filename (keys %Title) {
    my $best=ChooseLanguage(keys %{$Title{$filename}});
    $best='' if($best eq 'en');
    $BlogCache{$filename}{'Title'}=$Title{$filename}{$best};
    $BlogCache{$filename}{'Date'}=$Date{$filename}{$best};
    $BlogCache{$filename}{'Language'}=$best;
    push @BlogIndex,$best?"${filename}_$best":$filename;
  }
}

sub RebuildBlogCache {
  my (%Date,%Title,%WordCache);
  foreach my $file (glob("$Paths::File{'docs'}/weblog/*.box")) {
    next unless($file=~m!^/.*/(\d+(?:_[a-z\d\-]+)?)\.box$!);
    my $blog=$1; # non-localized ID
    my($box)=load DocBox("weblog/$blog",1); # don't translate
    next unless($box->Success); # FIXME - this should cause an error message
  
    my $text=lc HTMLToText($box->Title.' '.$box->Text);
    $text=~s/[\n\x{2014}]/ /g; # replace newline or em-dash with space (this should break words)
    $text=~s/[^ \-\d\w\xc0-\x{ffff}]//g; # strip punctuation characters (except hyphen)
    $text=~s/\s\-+/ /g; # strip leading hyphens
    $text=~s/\-+\s/ /g; # strip trailing hyphens 
    $text=~s/\s+/ /g; # consolidate whitespace
    $WordCache{$_}{$blog}=1 foreach (split / /,$text);
    my($filename,$language)=split /_/,$blog;
    $language='' unless(defined $language);
    $Title{$filename}{$language}=$box->Title; # $language could be ""
    $Date{$filename}{$language}=$box->Date;
  }
  open(CACHE,">$Paths::File{'data'}/blogwordcache.data") or Abort('Cannot write to blogwordcache');
  foreach my $word (sort keys %WordCache) {
    print CACHE "$word: ".join(',',sort keys %{$WordCache{$word}})."\n";
  }
  close CACHE;
  open(CACHE,">$Paths::File{'data'}/blogmetacache.data") or Abort('Cannot write to blogmetacache');
  foreach my $filename (sort {$a <=> $b} keys %Title) {
    foreach my $language (sort keys %{$Title{$filename}}) {
      my $local=$filename;
      $local.='_'.$language if($language);
      print CACHE "$local\t$Date{$filename}{$language}\t$Title{$filename}{$language}\n";
    }
  }
  close CACHE;
}

sub _GetBlogNav {
  # Arguments: current blog number
  # Returns: Box object
  
  my($blog)=@_;
  my @list=GetBlogList();
  my($first,$last,$next,$prev)=(-1,-1,-1,-1);
  my %Links=();
  if(@list) {
    $first=$list[0];
    $last=$list[$#list];
  }
  for(my $i=0;$i<@list;$i++) {
    if($list[$i]==$blog) {
      $prev=$list[$i-1] if($i>0);
      $next=$list[$i+1] if($i<$#list);
    }
  }
  my $html='<div class="blognav">';
  if($first>-1 && $first!=$blog) {
    $html.='{LINKTO:weblog-'.$first.'}<img src="{BASEPATH}/graphics/blognav/first.gif" style="width: 38px;" alt="First"></a>'."\n";
    $Links{'LinkFirst'}='weblog-'.$first;
  } else {
    $html.='<img src="{BASEPATH}/graphics/blognav/first-disabled.gif" style="width: 38px;" alt="First">'."\n";
  }
  if($prev>-1) {
    $html.='{LINKTO:weblog-'.$prev.'}<img src="{BASEPATH}/graphics/blognav/previous.gif" style="width: 27px;" alt="Previous"></a>'."\n";
    $Links{'LinkPrev'}='weblog-'.$prev;
  } else {
    $html.='<img src="{BASEPATH}/graphics/blognav/previous-disabled.gif" style="width: 27px;" alt="Previous">'."\n";
  }
  $html.='{LINKTO:weblog}<img src="{BASEPATH}/graphics/blognav/up.gif" style="width: 49px;" alt="Index"></a>'."\n";
  $Links{'LinkUp'}='weblog';
  if($next>-1) {
    $html.='{LINKTO:weblog-'.$next.'}<img src="{BASEPATH}/graphics/blognav/next.gif" style="width: 25px;" alt="Next"></a>'."\n";
    $Links{'LinkNext'}='weblog-'.$next;
  } else {
    $html.='<img src="{BASEPATH}/graphics/blognav/next-disabled.gif" style="width: 25px;" alt="Next">'."\n";
  }
  if($last>-1 && $last!=$blog) {
    $html.='{LINKTO:weblog-'.$last.'}<img src="{BASEPATH}/graphics/blognav/last.gif" style="width: 36px;" alt="Last"></a>'."\n";
    $Links{'LinkLast'}='weblog-'.$last;
  } else {
    $html.='<img src="{BASEPATH}/graphics/blognav/last-disabled.gif" style="width: 36px;" alt="Last">'."\n";
  }
  chop($html);
  $html.="</div>\n";
  return new DocBox('Title'=>Translate('Weblog Navigation'),'Class'=>'WhiteBox','Body'=>$html),%Links;
}


1;
