package Navigation;
#
# phroggy.com 5.3
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These functions relate to links between different pages within the site.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    GetPageCache
    GetURI
    GetDescription
    GetShortTitle
    GetIcon
    LinkTo
  );
}

use strict;
use warnings;
use File::Basename;
use Paths;
use Localize;
use PhroggyIO;
use Blog;

our %PageCache=();

sub GetPageCache {
  # Arguments: force refresh?
  
  my($force)=@_;
  unless(%PageCache && !$force) { # skip if already loaded
    my $cacheage=0; # if no cache file exists, always create one
    if(-f "$Paths::File{'data'}/pagecache.data") {
      $cacheage=(stat "$Paths::File{'data'}/pagecache.data")[9];
    }
    my $dirage=(stat $Paths::File{'docs'})[9];
    if($dirage>$cacheage || $force) { # cache is out of date
      _BuildPageCache();
    } else { # cache is up to date
      _ReadPageCache();
    }
  }
}

sub GetURI {
  # Arguments: filename
  # Returns: URI of that page

  my($filename)=@_;
  $filename='home' unless($filename);
  my $uri="/pages/$filename.html"; # default, should always work
  if($filename=~/^(.*?)\-(.*)$/) { # for /articles, /weblog
    $uri="/$1/$2.html";
  }
  GetPageCache();
  if(exists $PageCache{$filename} && $PageCache{$filename}->{'Path'} ne '') {
    $uri=$PageCache{$filename}->{'Path'};
  }
  return $Paths::Web{'base'}.$uri;
}

sub GetDescription {
  # Arguments: filename
  # Returns: Description, or undef if none
  
  my($filename)=@_;
  if($filename=~/^weblog-(.*)$/) {
    my($title,$date)=GetBlogInfo($1);
    return undef unless($title); # the blog entry doesn't exist
    $date=ShortDate($date);
    return Translate('Weblog entry').$1.": $title ($date)";
  }  
  GetPageCache();
  my $localized=LocalizeFilename($filename,keys %PageCache);
  return undef unless($localized); # the page doesn't exist in cache
  return $PageCache{$localized}->{'Description'};
}

sub GetShortTitle {
  # Arguments: filename
  # Returns: Short title to use for link text in navigation menu etc.
  
  my($filename)=@_;
  if($filename=~/^weblog-(.*)$/) {
    my($title)=GetBlogInfo($1);
    return $title?$title:'<span style="color: #fff; background-color: #800; font-weight: bold;">BROKEN LINK</span>';
  }  
  GetPageCache();
  my $localized=LocalizeFilename($filename,keys %PageCache);
  unless($localized) { # the page doesn't exist in cache
    return '<span style="color: #fff; background-color: #800; font-weight: bold;">BROKEN LINK</span>';
  }
  my $title=$PageCache{$localized}->{'ShortTitle'};
  return $title?$title:$filename; # if no short title was given, use the filename instead
}

sub GetIcon {
  # Arguments: filename
  # Returns: Icon, if any, or "file.gif"
  
  my($filename)=@_;
  my $default='file.gif';
  if($filename=~/^weblog-(.*)$/) {
    return $default; # blog entries don't have custom icons
  }  
  GetPageCache();
  my $localized=LocalizeFilename($filename,keys %PageCache);
  return $default unless($localized); # the page doesn't exist in cache
  my $icon=$PageCache{$localized}->{'Icon'};
  return $icon?$icon:$default; # use default if no icon was specified
}

sub LinkTo {
  # Arguments: filename, [theme, [background]]
  # Returns: anchor tag linking to that page

  my($f,$t,$b)=@_;
  my $q='';
  my $string='';
  if($t) {
    $q="?theme=$t";
    if($b) {
      $q.="\&background=$b";
    }
    $string='<a href="'.GetURI($f).$q."\">";
  } else {
    my $desc=GetDescription($f);
    if($desc) {
      $string='<a href="'.GetURI($f).$q.'" title="'.GetDescription($f).'">';
    } else { # this page has no description (or maybe doesn't exist at all)
      $string='<a href="'.GetURI($f).$q.'">';
    }
  }
  return $string;
}

sub _BuildPageCache {
  my @files=glob("$Paths::File{'docs'}/*.page");
  if(@files<5) {
    Abort('Too few page files are present.');
  }
  open(DB,">$Paths::File{'data'}/pagecache.data") or Abort('Cannot write to pagecache');
  %PageCache=();
  foreach my $path (@files) {
    my $filename=basename($path,'.page');
    open(PAGE,$path) or Abort("Cannot read $path");
    my %info=('Title'=>'','ShortTitle'=>'','Path'=>'','Description'=>'','Icon'=>'');
    while(<PAGE>) {
      chomp;
      $info{'Title'}=$1 if(/^TITLE:(.*)$/);
      $info{'ShortTitle'}=$1 if(/^SHORTTITLE:(.*)$/);
      $info{'Path'}=$1 if(/^PATH:(.*)$/);
      $info{'Description'}=$1 if(/^DESCRIPTION:(.*)$/);
      $info{'Icon'}=$1 if(/^ICON:(.*)$/);
    }
    close PAGE;
    print DB "$filename\t$info{'Title'}\t$info{'ShortTitle'}\t$info{'Path'}\t$info{'Description'}\t$info{'Icon'}\n";
    $PageCache{$filename}=\%info;
  }
  close DB;
}

sub _ReadPageCache {
  %PageCache=();
  open(DB,"$Paths::File{'data'}/pagecache.data") or Abort('Cannot read page cache.');
  while(<DB>) {
    chomp;
    my($filename,$title,$short,$path,$desc,$icon)=split "\t";
    $PageCache{$filename}={'Title'=>$title,'ShortTitle'=>$short,'Path'=>$path,'Description'=>$desc,'Icon'=>$icon};
  }
  close DB;
}


1;
