package PhroggyHTML;
#
# phroggy.com 5.4
#
# (C) 2009 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These functions deal with HTML parsing and character escaping.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    HTMLToText
    Escape
  );
}

use strict;
use warnings;

# http://www.natural-innovations.com/boo/doc-charset.html
our %Entities=(
  'quot'   => 34,
  'amp'    => 38,
  'apos'   => 39,
  'lt'     => 60,
  'gt'     => 62,
  'nbsp'   => 160,
  'Agrave' => 192,
  'agrave' => 224,
  'Aacute' => 193,
  'aacute' => 225,
  'Acirc'  => 194,
  'acirc'  => 226,
  'Atilde' => 195,
  'atilde' => 227,
  'Auml'   => 196,
  'auml'   => 228,
  'Aring'  => 197,
  'aring'  => 229,
  'AElig'  => 198,
  'aelig'  => 230,
  'Ccedil' => 199,
  'ccedil' => 231,
  'ETH'    => 208,
  'eth'    => 240,
  'Egrave' => 200,
  'egrave' => 232,
  'Eacute' => 201,
  'eacute' => 233,
  'Ecirc'  => 202,
  'ecirc'  => 234,
  'Euml'   => 203,
  'euml'   => 235,
  'Igrave' => 204,
  'igrave' => 237,
  'Iacute' => 205,
  'iacute' => 237,
  'Icirc'  => 206,
  'icirc'  => 238,
  'Iuml'   => 207,
  'iuml'   => 239,
  'micro'  => 181,
  'Ntilde' => 209,
  'ntilde' => 241,
  'Ograve' => 210,
  'ograve' => 242,
  'Oacute' => 211,
  'oacute' => 243,
  'Ocirc'  => 212,
  'ocirc'  => 244,
  'Otilde' => 213,
  'otilde' => 245,
  'Ouml'   => 214,
  'ouml'   => 246,
  'Oslash' => 216,
  'oslash' => 248,
  'szlig'  => 223,
  'THORN'  => 222,
  'thorn'  => 254,
  'Ugrave' => 217,
  'ugrave' => 249,
  'Uacute' => 218,
  'uacute' => 250,
  'Ucirc'  => 219,
  'ucirc'  => 251,
  'Uuml'   => 220,
  'uuml'   => 252,
  'Yacute' => 221,
  'yacute' => 253,
  'yuml'   => 255,
  'uml'    => 168,
  'macr'   => 175,
  'acute'  => 180,
  'cedil'  => 184,
  'iexcl'  => 161,
  'iquest' => 191,
  'middot' => 183,
  'brvbar' => 166,
  'laquo'  => 171,
  'raquo'  => 187,
  'para'   => 182,
  'sect'   => 167,
  'copy'   => 169,
  'reg'    => 174,
  'sup1'   => 185,
  'sup2'   => 178,
  'sup3'   => 179,
  'shy'    => 173,
  'times'  => 215,
  'divide' => 247,
  'frac14' => 188,
  'frac12' => 189,
  'frac34' => 190,
  'ordf'   => 170,
  'ordm'   => 186,
  'not'    => 172,
  'deg'    => 176,
  'plusmn' => 177,
  'curren' => 164,
  'cent'   => 162,
  'pound'  => 163,
  'yen'    => 165,
  'Delta'  => 916,
  'fnof'   => 402,
  'Omega'  => 937,
  'OElig'  => 338,
  'oelig'  => 339,
  'Scaron' => 352,
  'scaron' => 353,
  'Yuml'   => 376,
  'inodot' => 305,
  'circ'   => 710,
  'caron'  => 711,
  'breve'  => 728,
  'dot'    => 729,
  'ring'   => 730,
  'ogon'   => 731,
  'tilde'  => 732,
  'dblac'  => 733,
  'ndash'  => 8211,
  'mdash'  => 8212,
  'dagger' => 8224,
  'Dagger' => 8225,
  'bull'   => 8226,
  'hellip' => 8230,
  'lsquo'  => 8216,
  'rsquo'  => 8217,
  'lsquor' => 8218,
  'ldquo'  => 8220,
  'rdquo'  => 8221,
  'ldquor' => 8222,
  'lsaquo' => 8249,
  'rsaquo' => 8250,
  'trade'  => 8482,
  'radic'  => 8730,
  'infin'  => 8734,
  'int'    => 8747,
  'part'   => 8706,
  'ap'     => 8773,
  'ne'     => 8800,
  'le'     => 8804,
  'ge'     => 8805,
  'sum'    => 931,
  'permil' => 8240,
  'prod'   => 8719,
  'pi'     => 960,
  'loz'    => 9674,
);

sub HTMLToText {
  # Arguments: HTML
  # Returns: plain text, no formatting, one line per paragraph

  my($text)=@_;
  
  # Consolidate whitespace
  $text=~s/\s+/ /g;

  # Convert line-breaking tags
  $text=~s/<(?:p|div|ol|ul|blockquote) ?[^>]*>/\n\n/ig;
  $text=~s/<br ?[^>]*>/\n/ig;

  # Strip other HTML tags and phroggy.com markup codes
  $text=~s/<.*?>//g;
  $text=~s/{[A-Z][^}]+}//g;

  # Convert HTML entities into actual characters
  $text=~s/&#x([\da-f]+);/chr(hex($1))/ige;
  $text=~s/&#(\d+);/chr($1)/ge;
  $text=~s/&(\w{2,6});/exists($Entities{$1})?chr($Entities{$1}):"\&$1;"/ge;

  # Consolidate spaces again, and strip leading and trailing spaces from each line
  $text=~s/ +/ /g;
  $text=~s/^\s+//gm;
  $text=~s/\s+$//gm;
  
  return $text;
}

sub Escape {
  # Arguments: plain text
  # Returns: text with special characters escaped
  
  my($html)=@_;
  my $text='';
  for(my $x=0;$x<length($html);$x++) {
    my $entity=substr($html,$x,1);
    my $num=ord $entity;
    $entity='&amp;' if($num==38);
    $entity='&lt;' if($num==60);
    $entity='&gt;' if($num==62);
    $entity='&quot;' if($num==34);
    if(($num<32 || $num>126) && $num!=10) {
      $entity='&#'.$num.';';
      if($num<256) {
		foreach my $key (keys %Entities) {
		  if($Entities{$key}==$num) {
			$entity='&'.$key.';';
			last;
		  }
		}
	  }
    }
    $text.=$entity;
  }
  return $text;
}


1;
